#include "stats.h" 
#include "gsl/gsl_linalg.h"
#include "gsl/gsl_permutation.h"
#include "gsl/gsl_permute.h"
#include "gsl/gsl_cdf.h"
#include "gsl/gsl_statistics.h"
extern "C"{
#include "gsl/gsl_randist.h"
}
#include "gsl/gsl_rng.h"
#include "gsl/gsl_eigen.h"

#include <gsl/gsl_errno.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_roots.h>

void get_inverse(gsl_matrix *A, gsl_matrix *inverse)
{
    int n = A->size1;
    gsl_matrix *tmpA = gsl_matrix_alloc(n, n);
    gsl_matrix_memcpy(tmpA, A);
    gsl_permutation *p = gsl_permutation_alloc(n);
    int sign = 0;
    gsl_linalg_LU_decomp(tmpA, p, &sign);
    gsl_linalg_LU_invert(tmpA, p, inverse);
    gsl_permutation_free(p);
    gsl_matrix_free(tmpA);
}//a function to get inverse matrix

double get_logdet(gsl_matrix *A)
{
    double det=0.0;
    int n=A->size1;
    gsl_permutation *p=gsl_permutation_calloc(n);
    gsl_matrix *tmpA = gsl_matrix_calloc(n, n);
    int signum;
    gsl_matrix_memcpy(tmpA,A);
    gsl_linalg_LU_decomp(tmpA,p,&signum);
    det=gsl_linalg_LU_det(tmpA,signum);
    gsl_permutation_free(p);
    gsl_matrix_free(tmpA);
    return log(det);
}//a function to get det_value

double bf_matrix_normal(int nn, int pp, int dd, double ** mph, double ** mgt, double * beta, int nc, double ** cov)
{

	gsl_matrix * my = gsl_matrix_alloc(nn, dd); 
	for (int k = 0; k < dd; k++)
	{
		double sum = 0; 
		for (int i = 0; i < nn; i++)
			sum += mph[i][k]; 
		sum /= nn; 
		for (int i = 0; i < nn; i++)
			gsl_matrix_set(my, i, k, mph[i][k]-sum); 
	}
         
	gsl_matrix * mw = NULL;  
	if(nc > 0) {
		mw = gsl_matrix_alloc(nn, nc+1); 
		for (int k = 0; k < nc; k++)
			for (int i = 0; i < nn; i++) 
				gsl_matrix_set(mw, i, k, cov[i][k]);                 
		for (int i = 0; i < nn; i++)
			gsl_matrix_set(mw, i, nc, 1); 
		nc += 1; 

//		gsl_matrix * mV = gsl_matrix_alloc(nc+1,nc+1); 
//		gsl_vector * vS = gsl_vector_alloc(nc+1); 
//		gsl_vector * vwork = gsl_vector_alloc(nc+1); 
//		gsl_linalg_SV_decomp (mw, mV, vS, vwork); 
//		int anc = nc+1; 
//		for (int k = 0; k < nc+1; k++)
//		{
//			if(gsl_vector_get(vS, nc-k) < 1e-10) 
//				anc--; 
//			else 
//				break; 
//		}
//		nc = anc; 
//		gsl_matrix_free(mV); 
//		gsl_vector_free(vS); 
//		gsl_vector_free(vwork); 
	}
	else if(nc == 0) 
	{
		nc = 1; 
		mw = gsl_matrix_alloc(nn, nc); 
		for (int i = 0; i < nn; i++)
			gsl_matrix_set(mw, i, 0, 1); 
	}
    //add 1 into control columns; 
	//use columns Q whose sigular values are nonzero as a new control convariates. 

	int np = nc + pp;  
	gsl_matrix * ms = gsl_matrix_alloc(nn, np); 
	for (int k = 0; k < nc; k++)
		for (int i = 0; i < nn; i++)
			gsl_matrix_set(ms, i, k, gsl_matrix_get(mw, i, k)); 
	for (int k = 0; k < pp; k++)
	{
		double sum = 0; 
		for (int i = 0; i < nn; i++)
			sum += mgt[i][k]; 
		sum /= nn; 
		for (int i = 0; i < nn; i++)
			gsl_matrix_set(ms, i, nc+k, mgt[i][k]-sum); 
	}
    //X=c(W, L); 

	
    gsl_matrix * myy=gsl_matrix_alloc(dd, dd);
	gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,my,my,0,myy);//t(Y)%*%Y; 

    gsl_matrix * mxx = gsl_matrix_alloc(np, np);
    gsl_matrix * m1=gsl_matrix_alloc(dd,np);
    gsl_matrix * m3=gsl_matrix_alloc(dd,dd);
    gsl_matrix * m4=gsl_matrix_alloc(dd,dd);
	gsl_matrix * ytx=gsl_matrix_alloc(dd,np);
	gsl_matrix * mk = gsl_matrix_alloc(np, np);
	gsl_matrix * minvxx = gsl_matrix_alloc(np,np);

	gsl_matrix * minvww = gsl_matrix_alloc(nc,nc);
	gsl_matrix * wtw = gsl_matrix_alloc(nc,nc); 
	gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,mw,mw,0,wtw);//wtw; 
	get_inverse(wtw,minvww);//(wtw)^-1
	double detww=get_logdet(wtw); 

	gsl_matrix *ytw = gsl_matrix_alloc(dd, nc); 
	gsl_matrix *ytw2 = gsl_matrix_alloc(dd, nc); 
	gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,my,mw,0,ytw);
	gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,ytw,minvww,0,ytw2);
    gsl_matrix * yy2=gsl_matrix_alloc(dd,dd);
	gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,ytw2,ytw,0,yy2);
	gsl_matrix_memcpy(m4, myy);
	gsl_matrix_sub(m4,yy2);  //bottom; 
	double  detyy=get_logdet(m4);              


	double pm = dd -1;
    vector<double> lbf;                                               
	lbf.push_back(0); 
	lbf.push_back(0); 
	vector<double> sa;
	sa.push_back(0.20); 
	sa.push_back(0.50); 

	for (unsigned r = 0; r < sa.size(); r++)
	{
		gsl_matrix_set_identity(mk);//mk=I
		gsl_matrix_scale(mk,(1/sa.at(r))*(1/sa.at(r)));//mk=K
		for (int k = 0; k < nc; k++)
			gsl_matrix_set(mk, k, k, 0); 
		double detk=-log(sa.at(r)) * 2.0 * pp;//det(K)

		gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,ms,ms,0,mxx);//mxx=crossprod(X)
		gsl_matrix_add(mxx,mk);//mxx=XX+K
		double  detxxk=get_logdet(mxx);//det(XX+K)              


		get_inverse(mxx,minvxx);//minvxx=solve(XX+K)

		gsl_blas_dgemm(CblasTrans,CblasNoTrans,1,my,ms,0,ytx); //m5=Yt X

		gsl_blas_dgemm(CblasNoTrans,CblasNoTrans,1,ytx,minvxx,0,m1);

		gsl_blas_dgemm(CblasNoTrans,CblasTrans,1,m1,ytx,0,m3);

		gsl_matrix_memcpy(m4, myy);
		gsl_matrix_sub(m4,m3);
		double detm4=get_logdet(m4);
		lbf.at(r)=(0.5*dd*(detww+detk-detxxk)+0.5*(nn+pm)*(detyy-detm4));
	}

    gsl_matrix_free(mk);
    gsl_matrix_free(mxx);
    gsl_matrix_free(minvxx);
    gsl_matrix_free(myy);
    gsl_matrix_free(m1);
    gsl_matrix_free(m3);
    gsl_matrix_free(m4);
    gsl_matrix_free(ytx);
    gsl_matrix_free(ms);
    gsl_matrix_free(my);
    gsl_matrix_free(mw);

	gsl_matrix_free(minvww); 
	gsl_matrix_free(wtw); 
	gsl_matrix_free(ytw); 
	gsl_matrix_free(ytw2); 
	gsl_matrix_free(yy2); 

	double logbf = max(lbf.at(0), lbf.at(1)); 
	logbf += log(exp(lbf.at(0)-logbf)+exp(lbf.at(1)-logbf));
	logbf -= log(2.0); 
	lbf.clear(); 
	sa.clear(); 
    return logbf;

}

void leading_eigenvec(int ni, int ns, double * vph, double ** mgt, double * vec)
{
	vector<int> index; 
	for (int i = 0; i < ni; i++)
	{
		if(fabs(vph[i] - 123456) > 0.1) index.push_back(i); 
	}
	ni = (int) index.size(); 

	gsl_vector * xres = gsl_vector_alloc(ni); 
	{
		gsl_matrix * L = gsl_matrix_alloc(ni, ns); 
//		gsl_matrix * Q = gsl_matrix_alloc(ni, ni); 
//		gsl_matrix * R = gsl_matrix_alloc(ni, ns); 
		gsl_matrix * T = gsl_matrix_alloc(ns, ns); //T=R R^t; 
		gsl_matrix * V = gsl_matrix_alloc(ns, ns); //T=R R^t; 

		for (int i = 0; i < ni; i++)
		{
			int ii = index.at(i); 
			for (int s = 0; s < ns; s++)
				gsl_matrix_set(L, i, s, mgt[ii][s]);
		}
		gsl_vector * tau = gsl_vector_alloc(ns); 
		gsl_vector * work = gsl_vector_alloc(ns); 
//		gsl_linalg_QR_decomp(L, tau); 
//		gsl_linalg_QR_unpack(L, tau, Q, R); 
//		gsl_matrix_view mR = gsl_matrix_submatrix(R, 0, 0, ns, ns); 
//		gsl_blas_dgemm(CblasNoTrans, CblasTrans, 1.0, &mR.matrix, &mR.matrix, 0.0, T); 
//		gsl_linalg_cholesky_decomp(T); //T > 0;
        gsl_linalg_SV_decomp_mod(L, T, V, tau, work);    
		gsl_vector_view vL = gsl_matrix_column(L, 0); 
//        gsl_matrix_view mQ = gsl_matrix_submatrix(Q, 0, 0, ni, ns); 
//		gsl_blas_dgemv(CblasNoTrans, 1.0, &mQ.matrix, &vT.vector, 0.0, xres); 
        gsl_vector_memcpy(xres, &vL.vector); 

		gsl_matrix_free(L); 
//		gsl_matrix_free(Q); 
//		gsl_matrix_free(R); 
		gsl_matrix_free(T); 
		gsl_matrix_free(V); 
		gsl_vector_free(tau); 
		gsl_vector_free(work); 

		double sum = 0; 
		for (int i = 0; i < ni; i++)
			sum += gsl_vector_get(xres, i); 
		sum /= ni; 
		gsl_vector_add_constant(xres, -sum); 
		gsl_vector_scale(xres, sqrt((double)ni)); 
	}
	for (int i = 0; i < ni; i++)
		vec[i] = gsl_vector_get(xres, i); 
	gsl_vector_free(xres); 
}

double lrt_logistic_QR(int ni, int ns, double * vph, double ** mgt, double * beta, int nc, double ** cov, double &df) 
{  
//	cout << ni << " === "; 
	vector<int> index; 
	for (int i = 0; i < ni; i++)
	{
		if(fabs(vph[i] - 123456) > 0.1) index.push_back(i); 
	}
	ni = (int) index.size(); 

		gsl_matrix * L = gsl_matrix_alloc(ni, ns); 
		gsl_matrix * T = gsl_matrix_alloc(ns, ns); //T=R R^t; 
		gsl_matrix * V = gsl_matrix_alloc(ns, ns); //T=R R^t; 

		for (int i = 0; i < ni; i++)
		{
			int ii = index.at(i); 
			for (int s = 0; s < ns; s++)
				gsl_matrix_set(L, i, s, mgt[ii][s]);
		}
		gsl_vector * tau = gsl_vector_alloc(ns); 
		gsl_vector * work = gsl_vector_alloc(ns); 
        gsl_linalg_SV_decomp_mod(L, T, V, tau, work);    

		int tns = 0; 
		for (int s = 0; s < ns; s++)
		{
			if(gsl_vector_get(tau, s) > 1e-6) 
				tns ++; 
		}                                           

		for (int s = 0; s < ns; s++)
		{
			gsl_vector_view vL = gsl_matrix_column(L,s); 
			gsl_vector_scale(&vL.vector, gsl_vector_get(tau,s));
		}

		gsl_matrix_free(T); 
		gsl_matrix_free(V); 
		gsl_vector_free(work); 
		gsl_vector_free(tau); 

//	cout << ni << endl;  

    ns = tns; 
	df = ns; 
	int col = 1 + nc + ns; 
	int c0 = 1 + nc; 

	gsl_vector * gph = gsl_vector_alloc(ni); 
	for (int i = 0; i < ni; i++)
	{
		if(vph[index.at(i)] > 0.5) 
			gsl_vector_set(gph, i, 1); 
		else 
			gsl_vector_set(gph, i, 0); 
	}

	gsl_matrix * gX = gsl_matrix_alloc(ni, col); 
	gsl_matrix * gX0 = gsl_matrix_alloc(ni, c0); 
	for (int i = 0; i < ni; i++)
	{
		int ii = index.at(i); 
		gsl_matrix_set(gX, i, 0, 1); 
		gsl_matrix_set(gX0, i, 0, 1); 
		for (int c = 0; c < nc; c++)
		{
			gsl_matrix_set(gX, i, c+1, cov[ii][c]); 
			gsl_matrix_set(gX0, i, c+1, cov[ii][c]); 
		}
		for (int s = 0; s < ns; s++)
			gsl_matrix_set(gX, i, nc+1+s, gsl_matrix_get(L, i, s));
	}
	vector<int> ().swap(index); 
	gsl_matrix_free(L); 

	double loglike1 = 0; 
	{
		gsl_vector * bold = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bold); 
		gsl_vector * bnew = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bnew); 
//		gsl_vector * varb = gsl_vector_alloc(col); 
		gsl_vector * xty = gsl_vector_alloc(col); 
		gsl_vector * pi = gsl_vector_alloc(ni); 
		gsl_matrix * xbar = gsl_matrix_alloc(ni, col); 
		gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
		gsl_matrix * mV = gsl_matrix_alloc(col, col); 
		gsl_vector * vS = gsl_vector_alloc(col); 
		gsl_vector * work = gsl_vector_alloc(col); 
		gsl_matrix * mwork = gsl_matrix_alloc(col, col); 


		double likeold = -1e100; 
		int iter = 0; 
		while(1) {
			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
			gsl_matrix_memcpy(xbar, gX); 
			for (int i = 0; i < ni; i++)
			{   
				double pp = gsl_vector_get(pi, i); 
				pp = 1 / (1 + exp(-pp)); 
				double ww = pp * (1 - pp); 
				gsl_vector_view vv = gsl_matrix_row(xbar, i); 
				gsl_vector_scale(&vv.vector, ww); 
				double yy = gsl_vector_get(gph, i); 
				gsl_vector_set(pi, i, yy - pp); 
			}

			gsl_blas_dgemv(CblasTrans, 1.0, gX, pi, 0.0, xty); 
			gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, xbar, 0.0, gXtX); 

			gsl_linalg_SV_decomp_mod(gXtX, mwork, mV, vS, work); 
			for (int i = 0; i < col; i++)
	   		if( gsl_vector_get(vS, i) < 1e-6) 
				gsl_vector_set(vS,i, 0); 
			gsl_linalg_SV_solve(gXtX, mV, vS, xty, bnew); 
//			gsl_vector_memcpy(bnew, xty); 
//            gsl_linalg_HH_svx (gXtX,  bnew); 
			gsl_vector_add(bold, bnew); 

//			cout << iter << " ::: " << likeold << " ::: "; 
//			for( int j = 0; j < col; j++) 
//				cout << gsl_vector_get(bnew, j) << " "; 
//			cout << endl; 

			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
        	double likenew = 0; 
			for (int i = 0; i < ni; i++)
			{                              
				double pp = gsl_vector_get(pi, i); 
				double tt = 1 / (1 + exp(-pp)); 
				if(gsl_vector_get(gph,i) == 1) 
					likenew += log(tt); 
				else 
					likenew += log(1-tt); 
			}

//			if(fabs(gsl_vector_get(bnew, 0)) < 1e-3)
			if(fabs(likeold-likenew) < 0.5 || iter > 100) 
				break;
			likeold = likenew; 
			iter ++; 
		}

		loglike1 = likeold; 

		for (int j = 0; j < col; j++)
			beta[j] = gsl_vector_get(bold, j); 


		gsl_matrix_free(mV);
		gsl_vector_free(vS); 
		gsl_vector_free(work); 
		gsl_matrix_free(mwork);
		gsl_matrix_free(gXtX);
		gsl_matrix_free(xbar);
		gsl_vector_free(bold);
		gsl_vector_free(bnew);
//		gsl_vector_free(varb);
		gsl_vector_free(pi); 
		gsl_vector_free(xty); 
	}

	col = c0; 
	gsl_matrix_free(gX);
	gX = gsl_matrix_alloc(ni, col); 
	gsl_matrix_memcpy(gX, gX0); 
	double loglike0 = 0; 
	{
		gsl_vector * bold = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bold); 
		gsl_vector * bnew = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bnew); 
//		gsl_vector * varb = gsl_vector_alloc(col); 
		gsl_vector * xty = gsl_vector_alloc(col); 
		gsl_vector * pi = gsl_vector_alloc(ni); 
		gsl_matrix * xbar = gsl_matrix_alloc(ni, col); 
		gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
		gsl_matrix * mV = gsl_matrix_alloc(col, col); 
		gsl_vector * vS = gsl_vector_alloc(col); 
		gsl_vector * work = gsl_vector_alloc(col); 
		gsl_matrix * mwork = gsl_matrix_alloc(col, col); 

		int iter = 0; 
		double likeold = -1e100; 
		while(1) {
			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
			gsl_matrix_memcpy(xbar, gX); 
			for (int i = 0; i < ni; i++)
			{   
				double pp = gsl_vector_get(pi, i); 
				pp = 1/ (1 + exp(-pp)); 
				double ww = pp * (1 - pp); 
//				if(ww < 1e-6) ww = 1e-6; 
//				else if (ww > 1-1e-6) ww = 1-1e-6; 
				gsl_vector_view vv = gsl_matrix_row(xbar, i); 
				gsl_vector_scale(&vv.vector, ww); 
				double yy = gsl_vector_get(gph, i); 
				gsl_vector_set(pi, i, yy - pp); 
			}

			gsl_blas_dgemv(CblasTrans, 1.0, gX, pi, 0.0, xty); 
			gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, xbar, 0.0, gXtX); 
//			gsl_vector_view vdiag = gsl_matrix_diagonal(gXtX); 
//			gsl_vector_memcpy(varb, &vdiag.vector); 

			gsl_linalg_SV_decomp_mod(gXtX, mwork, mV, vS, work); 
			gsl_linalg_SV_solve(gXtX, mV, vS, xty, bnew); 
//			gsl_vector_memcpy(bnew, xty); 
//            gsl_linalg_HH_svx (gXtX,  bnew); 
			gsl_vector_add(bold, bnew); 


			double likenew = 0; 
			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
			for (int i = 0; i < ni; i++)
			{                              
				double pp = gsl_vector_get(pi, i); 
				double tt = 1 / (1 + exp(-pp)); 
				if(gsl_vector_get(gph,i) == 1) 
					likenew += log(tt); 
				else 
					likenew += log(1-tt); 
			}

			if(fabs(likeold-likenew) < 0.5 || iter > 100) 
				break;
			likeold = likenew; 
			iter ++; 
		}
		loglike0 = likeold; 

		gsl_matrix_free(mV);
		gsl_vector_free(vS); 
		gsl_vector_free(work); 
		gsl_matrix_free(mwork);
		gsl_matrix_free(gXtX);
		gsl_matrix_free(xbar);
		gsl_vector_free(bold);
		gsl_vector_free(bnew);
//		gsl_vector_free(varb);
		gsl_vector_free(pi); 
		gsl_vector_free(xty); 
	}

	gsl_matrix_free(gX);
	gsl_matrix_free(gX0);
	gsl_vector_free(gph);
	
	return (2*(loglike1 - loglike0));  
}

double lrt_logistic(int ni, int ns, double * vph, double ** mgt, double * beta, int nc, double ** cov) 
{  
//	cout << ni << " === "; 
	vector<int> index; 
	for (int i = 0; i < ni; i++)
	{
		if(fabs(vph[i] - 123456) > 0.1) index.push_back(i); 
	}
	ni = (int) index.size(); 
//	cout << ni << endl;  
	if(ni == 0) return 1; 

	int col = 1 + nc + ns; 
	int c0 = 1 + nc; 
	//+1 because of adding a column of 1; 

	gsl_vector * gph = gsl_vector_alloc(ni); 

	double tm = 0; 
	for (int i = 0; i < ni; i++)
	   	tm += vph[index.at(i)]; 
    tm /= ni; 

	for (int i = 0; i < ni; i++)
	{
		if(vph[index.at(i)] > tm) 
			gsl_vector_set(gph, i, 1); 
		else 
			gsl_vector_set(gph, i, 0); 
	}

	
	gsl_matrix * gX = gsl_matrix_alloc(ni, col); 
	gsl_matrix * gX0 = gsl_matrix_alloc(ni, c0); 
	for (int i = 0; i < ni; i++)
	{
		int ii = index.at(i); 
		gsl_matrix_set(gX, i, 0, 1); 
		gsl_matrix_set(gX0, i, 0, 1); 
		for (int c = 0; c < nc; c++)
		{
			gsl_matrix_set(gX, i, c+1, cov[ii][c]); 
			gsl_matrix_set(gX0, i, c+1, cov[ii][c]); 
		}
		for (int s = 0; s < ns; s++)
		{
			gsl_matrix_set(gX, i, nc+1+s, mgt[ii][s]);
		}
	}
	vector<int> ().swap(index); 


	double loglike1 = 0; 
	{
		gsl_vector * bold = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bold); 
		gsl_vector * bnew = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bnew); 
//		gsl_vector * varb = gsl_vector_alloc(col); 
		gsl_vector * xty = gsl_vector_alloc(col); 
		gsl_vector * pi = gsl_vector_alloc(ni); 
		gsl_matrix * xbar = gsl_matrix_alloc(ni, col); 
		gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
		gsl_matrix * mV = gsl_matrix_alloc(col, col); 
		gsl_vector * vS = gsl_vector_alloc(col); 
		gsl_vector * work = gsl_vector_alloc(col); 
		gsl_matrix * mwork = gsl_matrix_alloc(col, col); 


		double likeold = -1e100; 
		int iter = 0; 
		while(1) {
			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
			gsl_matrix_memcpy(xbar, gX); 
			for (int i = 0; i < ni; i++)
			{   
				double pp = gsl_vector_get(pi, i); 
				pp = 1 / (1 + exp(-pp)); 
				double ww = pp * (1 - pp); 
				gsl_vector_view vv = gsl_matrix_row(xbar, i); 
				gsl_vector_scale(&vv.vector, ww); 
				double yy = gsl_vector_get(gph, i); 
				gsl_vector_set(pi, i, yy - pp); 
			}

			gsl_blas_dgemv(CblasTrans, 1.0, gX, pi, 0.0, xty); 
			gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, xbar, 0.0, gXtX); 

			gsl_linalg_SV_decomp_mod(gXtX, mwork, mV, vS, work); 
			for (int i = 0; i < col; i++)
	   		if( gsl_vector_get(vS, i) < 1e-6) 
				gsl_vector_set(vS,i, 0); 
			gsl_linalg_SV_solve(gXtX, mV, vS, xty, bnew); 
//			gsl_vector_memcpy(bnew, xty); 
//            gsl_linalg_HH_svx (gXtX,  bnew); 
			gsl_vector_add(bold, bnew); 

//			cout << iter << " ::: " << likeold << " ::: "; 
//			for( int j = 0; j < col; j++) 
//				cout << gsl_vector_get(bnew, j) << " "; 
//			cout << endl; 

			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
        	double likenew = 0; 
			for (int i = 0; i < ni; i++)
			{                              
				double pp = gsl_vector_get(pi, i); 
				double tt = 1 / (1 + exp(-pp)); 
				if(gsl_vector_get(gph,i) == 1) 
					likenew += log(tt); 
				else 
					likenew += log(1-tt); 
			}

//			if(fabs(gsl_vector_get(bnew, 0)) < 1e-3)
			if(fabs(likeold-likenew) < 1e-3 || iter > 100) 
				break;
			likeold = likenew; 
			iter ++; 
		}

		loglike1 = likeold; 

		for (int j = 0; j < col; j++)
			beta[j] = gsl_vector_get(bold, j); 


		gsl_matrix_free(mV);
		gsl_vector_free(vS); 
		gsl_vector_free(work); 
		gsl_matrix_free(mwork);
		gsl_matrix_free(gXtX);
		gsl_matrix_free(xbar);
		gsl_vector_free(bold);
		gsl_vector_free(bnew);
//		gsl_vector_free(varb);
		gsl_vector_free(pi); 
		gsl_vector_free(xty); 
	}

	col = c0; 
	gsl_matrix_free(gX);
	gX = gsl_matrix_alloc(ni, col); 
	gsl_matrix_memcpy(gX, gX0); 
	double loglike0 = 0; 
	{
		gsl_vector * bold = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bold); 
		gsl_vector * bnew = gsl_vector_alloc(col); 
		gsl_vector_set_zero(bnew); 
//		gsl_vector * varb = gsl_vector_alloc(col); 
		gsl_vector * xty = gsl_vector_alloc(col); 
		gsl_vector * pi = gsl_vector_alloc(ni); 
		gsl_matrix * xbar = gsl_matrix_alloc(ni, col); 
		gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
		gsl_matrix * mV = gsl_matrix_alloc(col, col); 
		gsl_vector * vS = gsl_vector_alloc(col); 
		gsl_vector * work = gsl_vector_alloc(col); 
		gsl_matrix * mwork = gsl_matrix_alloc(col, col); 

		int iter = 0; 
		double likeold = -1e100; 
		while(1) {
			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
			gsl_matrix_memcpy(xbar, gX); 
			for (int i = 0; i < ni; i++)
			{   
				double pp = gsl_vector_get(pi, i); 
				pp = 1/ (1 + exp(-pp)); 
				double ww = pp * (1 - pp); 
//				if(ww < 1e-6) ww = 1e-6; 
//				else if (ww > 1-1e-6) ww = 1-1e-6; 
				gsl_vector_view vv = gsl_matrix_row(xbar, i); 
				gsl_vector_scale(&vv.vector, ww); 
				double yy = gsl_vector_get(gph, i); 
				gsl_vector_set(pi, i, yy - pp); 
			}

			gsl_blas_dgemv(CblasTrans, 1.0, gX, pi, 0.0, xty); 
			gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, xbar, 0.0, gXtX); 
//			gsl_vector_view vdiag = gsl_matrix_diagonal(gXtX); 
//			gsl_vector_memcpy(varb, &vdiag.vector); 

			gsl_linalg_SV_decomp_mod(gXtX, mwork, mV, vS, work); 
			gsl_linalg_SV_solve(gXtX, mV, vS, xty, bnew); 
//			gsl_vector_memcpy(bnew, xty); 
//            gsl_linalg_HH_svx (gXtX,  bnew); 
			gsl_vector_add(bold, bnew); 


			double likenew = 0; 
			gsl_blas_dgemv(CblasNoTrans, 1.0, gX, bold, 0.0, pi); 
			for (int i = 0; i < ni; i++)
			{                              
				double pp = gsl_vector_get(pi, i); 
				double tt = 1 / (1 + exp(-pp)); 
				if(gsl_vector_get(gph,i) == 1) 
					likenew += log(tt); 
				else 
					likenew += log(1-tt); 
			}

			if(fabs(likeold-likenew) < 1e-3 || iter > 100) 
				break;
			likeold = likenew; 
			iter ++; 
		}
		loglike0 = likeold; 

		gsl_matrix_free(mV);
		gsl_vector_free(vS); 
		gsl_vector_free(work); 
		gsl_matrix_free(mwork);
		gsl_matrix_free(gXtX);
		gsl_matrix_free(xbar);
		gsl_vector_free(bold);
		gsl_vector_free(bnew);
//		gsl_vector_free(varb);
		gsl_vector_free(pi); 
		gsl_vector_free(xty); 
	}

	gsl_matrix_free(gX);
	gsl_matrix_free(gX0);
	gsl_vector_free(gph);
	
	return (2*(loglike1 - loglike0));  
}

struct mmparam {
	int p; 
	int q; 
	int n; 
	double yty; 
	gsl_vector * y;  
	gsl_vector * ytP;  
	gsl_vector * ytX;  
	gsl_matrix * P;  
	gsl_matrix * X;  
	gsl_matrix * PtP;  
	gsl_matrix * XtX;  
	gsl_matrix * XtP;  
	double like; 
	double df; 
};  

double dfdb(double b2, void * paramp) 
{  
	struct mmparam * mmp = (struct mmparam *) paramp; 

	int p = mmp->p; 
	int q = mmp->q; 
	int n = mmp->n; 
	int c = q; 
	double yty = mmp->yty; 
	gsl_vector * y = mmp->y; 
	gsl_vector * ytP = mmp->ytP; 
	gsl_vector * ytX = mmp->ytX; 
	gsl_matrix * P = mmp->P; 
	gsl_matrix * X = mmp->X;
	gsl_matrix * PtP = mmp->PtP; 
	gsl_matrix * XtX = mmp->XtX; 
	gsl_matrix * XtP = mmp->XtP; 
	
	gsl_matrix * Ip = gsl_matrix_alloc(p,p);  
	gsl_matrix_set_identity(Ip); 

	gsl_vector * vn1 = gsl_vector_alloc(n); 
	gsl_vector * vn2 = gsl_vector_alloc(n); 
	gsl_vector * vp1 = gsl_vector_alloc(p); 
	gsl_vector * vp2 = gsl_vector_alloc(p); 
//	gsl_vector * vp3 = gsl_vector_alloc(p); 
	gsl_vector * vp4 = gsl_vector_alloc(p); 
	gsl_vector * vc1 = gsl_vector_alloc(c); 
	gsl_vector * vc2 = gsl_vector_alloc(c); 
	gsl_vector * vc3 = gsl_vector_alloc(c); 

	gsl_matrix * M = gsl_matrix_alloc(p, p); 
	gsl_matrix * invM = gsl_matrix_alloc(p, p); 
	gsl_matrix * invX = gsl_matrix_alloc(c, c); 
	gsl_matrix * Mcp = gsl_matrix_alloc(c, p); 
	gsl_matrix * Mpp = gsl_matrix_alloc(p, p); 

	{
		gsl_matrix_memcpy(M, PtP); 
		gsl_matrix_scale(M, b2); 
		gsl_matrix_add(M, Ip); 
		//M=Iq+b2 PtP; 
		
		double logdetH = 0; 
		gsl_matrix_memcpy(invM, M); 
		gsl_linalg_cholesky_decomp(invM); 
		for (int i = 0; i < p; i++)
			logdetH += 2.0 * log(gsl_matrix_get(invM, i, i)); 
		gsl_linalg_cholesky_invert(invM); 
		
		double tr1 = 0; 
		for (int i = 0; i < p; i++)
			tr1 += gsl_matrix_get(PtP, i, i); 
		gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, PtP, invM, 0.0, Mpp); 
		gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, Mpp, PtP, 0.0, M); 
		double tr2 = 0; 
		for (int i = 0; i < p; i++)
			tr2 += gsl_matrix_get(M, i,i); 
		tr1 -= b2 * tr2; 
		//trace(Hi S); M destroyed. 

		gsl_blas_dgemm(CblasNoTrans, CblasNoTrans, 1.0, XtP, invM, 0.0, Mcp); 
		gsl_blas_dgemm(CblasNoTrans, CblasTrans, 1.0, Mcp, XtP, 0.0, invX); 
		gsl_matrix_scale(invX, -b2); 
		gsl_matrix_add(invX, XtX); 
		gsl_linalg_cholesky_decomp(invX); 
		gsl_linalg_cholesky_invert(invX); 

		double s1 = 0; 
		gsl_blas_dgemv(CblasNoTrans, 1.0, invM, ytP, 0.0, vp1); 
		gsl_blas_ddot(vp1, ytP, &s1); 
		s1 = yty - b2 * s1; 
		double s2 = 0; 
		gsl_blas_dgemv(CblasNoTrans, 1.0, XtP, vp1, 0.0, vc1); 
		gsl_vector_scale(vc1, -b2); 
		gsl_vector_add(vc1, ytX); 
		gsl_vector_memcpy(vc3, vc1); 
		//vc3 = yHiX; 
		gsl_blas_dgemv(CblasTrans, 1.0, invX, vc1, 0.0, vc2); 
		gsl_blas_ddot(vc1, vc2, &s2); 
		double ytQy = s1 - s2; 

		mmp->like = -0.5 * logdetH - 0.5 * n * log(ytQy); 
                
		gsl_blas_dgemv(CblasNoTrans, 1.0, invX, vc3, 0.0, vc1); 
		gsl_blas_dgemv(CblasNoTrans, 1.0, X, vc1, 0.0, vn1); 
		gsl_vector_scale(vn1, -1.0); 
		gsl_vector_add(vn1, y); 

		gsl_blas_dgemv(CblasTrans, 1.0, P, vn1, 0.0, vp1); 
		gsl_blas_dgemv(CblasNoTrans, 1.0, invM, vp1, 0.0, vp2); 
		gsl_blas_dgemv(CblasNoTrans, 1.0, PtP, vp2, 0.0, vp4); 
		gsl_vector_scale(vp4, -b2); 
		gsl_vector_add(vp4, vp1); 
		//vp4 = yQP

		double ytQSQy = 0; 
		gsl_blas_ddot(vp4, vp4, &ytQSQy); 

		mmp->df = -0.5 * tr1 + 0.5 * n * ytQSQy / ytQy; 
	}
	
	gsl_matrix_free(M); 
	gsl_matrix_free(Mpp); 
	gsl_matrix_free(Mcp); 
	gsl_matrix_free(invM); 
	gsl_matrix_free(invX); 
	gsl_matrix_free(Ip); 

	gsl_vector_free(vn1); 
	gsl_vector_free(vn2); 
	gsl_vector_free(vp1); 
	gsl_vector_free(vp2); 
//	gsl_vector_free(vp3); 
	gsl_vector_free(vp4); 
	gsl_vector_free(vc1); 
	gsl_vector_free(vc2); 
	gsl_vector_free(vc3); 

	return (mmp->df); 
}

double lrt_mixed_model(int ni, int ns, double * vph, double ** mgt, double * beta, int nc, double ** cov) 
{  
	int p = ns; 
	int q = nc + 1; 
	int n = ni; 
	int c = q; 

	gsl_vector * y = gsl_vector_alloc(n); 
	center(vph, ni); 
	for (int i = 0; i < n; i++)
   		gsl_vector_set(y, i, vph[i]); 
   
	gsl_matrix * P = gsl_matrix_alloc(n, p); 
	gsl_matrix * X = gsl_matrix_alloc(n, c); 
	for (int i = 0; i < ni; i++)
	{
		gsl_matrix_set(X, i, 0, 1); 
		for (int j = 0; j < nc; j++)
			gsl_matrix_set(X, i, 1+j, cov[i][j]); 
		for (int s = 0; s < p; s++)
		{
			gsl_matrix_set(P, i, s, mgt[i][s]);
		}
	}

	struct mmparam * mmp = new (struct mmparam); 

	mmp->n = n; 
	mmp->p = p; 
	mmp->q = q; 
	mmp->y = gsl_vector_alloc(n); 
	mmp->ytP = gsl_vector_alloc(p); 
	mmp->ytX = gsl_vector_alloc(c); 
	mmp->P = gsl_matrix_alloc(n, p); 
	mmp->X = gsl_matrix_alloc(n, c); 
	mmp->PtP = gsl_matrix_alloc(p, p); 
	mmp->XtX = gsl_matrix_alloc(c, c); 
	mmp->XtP = gsl_matrix_alloc(c, p); 

	gsl_blas_ddot(y, y, &(mmp->yty)); 
	gsl_vector_memcpy(mmp->y, y); 
	gsl_blas_dgemv(CblasTrans, 1.0, P, y, 0.0, mmp->ytP); 
	gsl_blas_dgemv(CblasTrans, 1.0, X, y, 0.0, mmp->ytX); 
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, P, P, 0.0, mmp->PtP); 
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, X, X, 0.0, mmp->XtX); 
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, X, P, 0.0, mmp->XtP); 

	gsl_matrix_memcpy(mmp->P, P); 
	gsl_matrix_memcpy(mmp->X, X); 


	vector<pair<double, double> > vb_lh;
	double x_lo = 1e-5, x_hi = 1e5;
	double b2 = 0; 
    int nreg = 10; 
	double b_interval = log(x_hi/x_lo) / (double) nreg; 
	for (int i = 0; i < nreg; i++)
	{
		double b_lo = x_lo * exp(b_interval*i); 
		double b_hi = x_lo * exp(b_interval*(i+1)); 

		dfdb(b_lo, (void*) mmp); 
		double dev_lo = mmp->df; 
		dfdb(b_hi, (void*) mmp); 
		double dev_hi = mmp->df; 
        if(dev_lo * dev_hi < 0) {
			vb_lh.push_back(pair<double, double>(b_lo, b_hi)); 
		}
	}

	if(vb_lh.empty()) {
		dfdb(x_lo, (void*) mmp); 
		double logf_lo = mmp->like; 
		dfdb(x_hi, (void*) mmp); 
		double logf_hi = mmp->like; 
		if(logf_lo >= logf_hi) 
			b2 = x_lo; 
		else
			b2 = x_hi; 
	}
	else 
	{
		const gsl_root_fsolver_type * T1 = gsl_root_fsolver_brent;
		gsl_root_fsolver * s1 = gsl_root_fsolver_alloc (T1);
		gsl_function F1;
		F1.function = &dfdb;
		F1.params = mmp;
		int status,iter;

		double logf = -1e100;
		for (vector<double>::size_type i=0; i < vb_lh.size(); ++i) {
			double b_lo = vb_lh.at(i).first; 
			double b_hi = vb_lh.at(i).second; 
			double bt; 

			iter=0;
			gsl_root_fsolver_set (s1, &F1, b_lo, b_hi);
			do{
				iter++;
				status = gsl_root_fsolver_iterate (s1);
				bt = gsl_root_fsolver_root (s1);
				b_lo = gsl_root_fsolver_x_lower (s1);
				b_hi = gsl_root_fsolver_x_upper (s1);
				status = gsl_root_test_interval (b_lo, b_hi,0.0, 0.001);
//				cout << iter << " " << bt << " " << b_lo << " " << b_hi << endl; 
			}   while(status== GSL_CONTINUE && iter<100);
                         
			dfdb(bt, (void*) mmp); 
			if(i == 0) {logf=mmp->like; b2 = bt;}
			else if(logf < mmp->like) {logf=mmp->like; b2=bt;}
			else {;}
		}
		gsl_root_fsolver_free(s1); 
    }
	
	////////////////////////////
    dfdb(b2, (void*) mmp);
	double l1 = mmp->like; 
    dfdb(0, (void*) mmp);
	double l0 = mmp->like; 

	double chisq = 2 * (l1 - l0);
	if(chisq < 0) chisq = 0; 
//	double pval = gsl_cdf_chisq_Q(chisq, p);

	gsl_vector_free(mmp->y); 
	gsl_vector_free(mmp->ytP); 
	gsl_vector_free(mmp->ytX); 
	gsl_matrix_free(mmp->P); 
	gsl_matrix_free(mmp->X); 
	gsl_matrix_free(mmp->PtP); 
	gsl_matrix_free(mmp->XtX); 
	gsl_matrix_free(mmp->XtP); 
	delete mmp;

	gsl_vector_free(y); 
	gsl_matrix_free(P); 
	gsl_matrix_free(X); 

	return(chisq); 
}

double bf_linear_QR(int ni, int ns, double * vph, double ** mgt, double * beta, int nc, double ** cov) 
{  
	vector<int> index; 
	for (int i = 0; i < ni; i++)
	{
		if(fabs(vph[i] - 123456) > 0.1) index.push_back(i); 
	}
	ni = (int) index.size(); 
	if(ni == 0 ) return 0; 

    //L is ni x ns; seek leading eigenvector of L L^t. 
	//first do QR decomposition of L = QR; 
	//then do SV decomp of R R^t = S D S^t; 
	//obtain QS, then the column corrresponding to the largest eigenvalue is the lead eigen vector. 
	//normalize this eigenvector, pass to the BF calculation; 
	
		gsl_matrix * L = gsl_matrix_alloc(ni, ns); 
		gsl_matrix * T = gsl_matrix_alloc(ns, ns); //T=R R^t; 
		gsl_matrix * V = gsl_matrix_alloc(ns, ns); //T=R R^t; 

		for (int i = 0; i < ni; i++)
		{
			int ii = index.at(i); 
			for (int s = 0; s < ns; s++)
				gsl_matrix_set(L, i, s, mgt[ii][s]);
		}
		gsl_vector * tau = gsl_vector_alloc(ns); 
		gsl_vector * work = gsl_vector_alloc(ns); 
        gsl_linalg_SV_decomp_mod(L, T, V, tau, work);    

		int tns = 0; 
		for (int s = 0; s < ns; s++)
		{
			if(gsl_vector_get(tau, s) > 1e-6) 
				tns ++; 
		}                                           


		for (int s = 0; s < ns; s++)
		{
			gsl_vector_view vL = gsl_matrix_column(L,s); 
			gsl_vector_scale(&vL.vector, gsl_vector_get(tau,s));
		}

		gsl_matrix_free(T); 
		gsl_matrix_free(V); 
		gsl_vector_free(work); 
		gsl_vector_free(tau); 

	ns = tns; 
    int col = 1 + nc + ns; 
	gsl_vector * gph = gsl_vector_alloc(ni); 
	for (int i = 0; i < ni; i++)
		gsl_vector_set(gph, i, vph[index.at(i)]); 
	gsl_matrix * gX = gsl_matrix_alloc(ni, col); 
	for (int i = 0; i < ni; i++)
	{
		int ii = index.at(i); 
		gsl_matrix_set(gX, i, 0, 1); 
		for (int c = 0; c < nc; c++)
			gsl_matrix_set(gX, i, c+1, cov[ii][c]); 
		for (int s = 0; s < ns; s++)
			gsl_matrix_set(gX, i, nc+1+s, gsl_matrix_get(L, i, s));
	}
    vector<int> ().swap(index); 
	gsl_matrix_free(L); 

	double mean = 0; 
	double var = 0; 
	for (int i = 0; i < ni; i++)
		mean += gsl_vector_get(gph, i); 
	mean /= ni; 
	for (int i = 0; i < ni; i++)
	{
		double temp = gsl_vector_get(gph, i) - mean; 
		var += temp * temp; 
	}
	if (fabs(var) < 1e-10) 
	{
		for (int i = 0; i < col; i++)
			beta[i] = 0; 
		return 0; 
	}
	double yy = 0; 
	gsl_blas_ddot(gph, gph, &yy); 
	
	gsl_matrix * ginvOmega = gsl_matrix_alloc(col, col); 
	gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, gX, 0.0, gXtX); 


	gsl_vector * gyx = gsl_vector_alloc(col); 
	gsl_blas_dgemv(CblasTrans, 1.0, gX, gph, 0.0, gyx); 
	gsl_vector * res = gsl_vector_alloc(col); 
	for (int i = 0; i < col; i++)
		beta[i] = 0;  
	
	vector<double> vsigma_a; 
//	vsigma_a.push_back(0.1); 
	vsigma_a.push_back(0.2); 
	vsigma_a.push_back(0.5); 

    vector<double> vsa;   
	for (unsigned r = 0; r < vsigma_a.size(); r++)
		vsa.push_back(1.0 / vsigma_a.at(r) / vsigma_a.at(r)); 

	////////////////////////////////////////////////////
//	cout << ni << " \t " << nc << " \t " << col << endl; 
//	cout << gX->size1 << "\t" << gX->size2 << endl; 
	gsl_matrix_view  mx = gsl_matrix_submatrix(gX,0,0, ni, nc+1); 
	gsl_matrix * xtx = gsl_matrix_alloc(nc+1, nc+1); 
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, &mx.matrix, &mx.matrix, 0.0, xtx); 
	gsl_linalg_cholesky_decomp(xtx); 
	double detnull = 0; 
	for (int i = 0; i < nc+1; i++)
		detnull += log(fabs(gsl_matrix_get(xtx, i, i))); 
	detnull *= 2.0; 
	gsl_vector * yx = gsl_vector_alloc(nc+1); 
	gsl_blas_dgemv(CblasTrans, 1.0, &mx.matrix, gph, 0.0, yx); 
	gsl_vector *xb = gsl_vector_alloc(nc+1); 
	gsl_linalg_cholesky_solve(xtx, yx, xb); //res = omega %*% gyx; 
	double tt = 0; 
	gsl_blas_ddot(xb, yx, &tt); 
	gsl_matrix_free(xtx); 
	gsl_vector_free(yx); 
	gsl_vector_free(xb); 

	int repeat = (int) vsa.size(); 
	gsl_vector * bf = gsl_vector_alloc(repeat); 
	for (unsigned p = 0; p < vsa.size(); p++)
	{
		gsl_matrix_memcpy(ginvOmega, gXtX); 
		double inv_va = vsa.at(p); 
		for (int i = 0; i < ns; i++)
		{
			double tmp = 0; 
			tmp = gsl_matrix_get(ginvOmega, i+nc+1, i+nc+1) + inv_va; 
			gsl_matrix_set(ginvOmega, i+nc+1, i+nc+1, tmp);
		}   // add prior on diagnal. 
		
		gsl_linalg_cholesky_decomp(ginvOmega); 
		double logdet = 0; 
		for (int i = 0; i < col; i++)
			logdet += log(fabs(gsl_matrix_get(ginvOmega, i, i))); 
		logdet *= 2.0; 
		
		//y^t X;
		gsl_linalg_cholesky_solve(ginvOmega, gyx, res); //res = omega %*% gyx; 
		
//		cout << "logdet: " << logdet << endl; 
		
//		cout << "gyx : " ; 
//		for (int j = 0; j < col; j++)
//			cout << gsl_vector_get(gyx, j) << " "; 
//		cout << endl; 
														  
		double bob = 0;
		gsl_blas_ddot(res, gyx, &bob); 
		
		double tau = sqrt((yy - bob) / ni); 
		for (int i = 0; i < col; i++)
			beta[i] += (gsl_vector_get(res, i) / tau); 
        /////////////////////////////////////////////////

		double ph_2 = gsl_vector_get(gyx, 0);
		ph_2 *= ph_2; 
		
		double tlast = (yy - bob) / (yy - tt);
//		cout << "tlast: " << tlast << endl; 
		
//		log1[p] = -ni/2.0 * log(yy-bob) - 0.5 * logdet + 0.5 * ns * log(inv_va); 
//		log0[p] = -ni/2.0 * log(yy-tt) - 0.5 * detnull; 

		double logvavdn = ns * log((double) inv_va) + detnull;    
		double logbf = -0.5 * logdet + 0.5 * logvavdn - ni * 0.5 * log((double)tlast); 
		gsl_vector_set(bf, p, logbf); 
//		cout << "lobgf: " << logbf << endl; 
	}

	for (int i = 0; i < col; i++)
		beta[i] /= repeat; 
	double maxbf = gsl_vector_max(bf); 
	gsl_vector_add_constant(bf, -maxbf); 
	double factor = 0; 
	for (int i = 0; i < repeat; i++)
		factor += exp(gsl_vector_get(bf, i)); 
	factor /= (double) repeat; 
	double logbf = maxbf + log(factor); 
		
	gsl_matrix_free(gX);
	gsl_vector_free(gph);
	gsl_matrix_free(gXtX);
	gsl_matrix_free(ginvOmega);
	gsl_vector_free(gyx);
	gsl_vector_free(res);
	gsl_vector_free(bf); 
	return (logbf);  
}

double bf_linear(int ni, int ns, double * vph, double ** mgt, double * beta, int nc, double ** cov) 
{  
	int col =  1 + nc + ns; 
	vector<int> index; 
	for (int i = 0; i < ni; i++)
	{
		if(fabs(vph[i] - 123456) > 0.1) 
			index.push_back(i); 
	}
	ni = (int) index.size(); 
	if(ni == 0) return 0; 
	//remove individuals marked by 123456 on phenotypes; this is not very safe. 

	gsl_vector * gph = gsl_vector_alloc(ni); 
	for (int i = 0; i < ni; i++)
		gsl_vector_set(gph, i, vph[index.at(i)]); 
	gsl_matrix * gX = gsl_matrix_alloc(ni, col); 
	for (int i = 0; i < ni; i++)
	{
		int ii = index.at(i); 
		gsl_matrix_set(gX, i, 0, 1); 
		for (int c = 0; c < nc; c++)
			gsl_matrix_set(gX, i, 1+c, cov[ii][c]); 
		for (int s = 0; s < ns; s++)
			gsl_matrix_set(gX, i, 1+nc+s, mgt[ii][s]);
	}
    vector<int> ().swap(index); 
	//prepare design matrix, adding the column of 1. 

	gsl_matrix * ginvOmega = gsl_matrix_alloc(col, col); 
	gsl_matrix * gXtX = gsl_matrix_alloc(col, col); 
	gsl_blas_dgemm(CblasTrans, CblasNoTrans, 1.0, gX, gX, 0.0, gXtX); 
	//xtx
	gsl_vector * gyx = gsl_vector_alloc(col); 
	gsl_blas_dgemv(CblasTrans, 1.0, gX, gph, 0.0, gyx); 
	//xty
	gsl_vector * res = gsl_vector_alloc(col); 
//	for (int i = 0; i < col; i++)
//		beta[i] = 0;  
	
	////////////////////////////////////////////////////
	gsl_matrix_view  mx = gsl_matrix_submatrix(gXtX,0,0, nc+1, nc+1); 
	gsl_matrix * xtx = gsl_matrix_alloc(nc+1, nc+1); 
	gsl_matrix_memcpy(xtx, &mx.matrix); 
	gsl_vector_view vx = gsl_vector_subvector(gyx, 0, nc+1); 
	gsl_vector * yx = gsl_vector_alloc(nc+1); 
	gsl_vector_memcpy(yx, &vx.vector); 
	gsl_vector *xb = gsl_vector_alloc(nc+1); 
	gsl_linalg_cholesky_decomp(xtx); 
	gsl_linalg_cholesky_solve(xtx, yx, xb); 
	//xb = inv(xtx) %*% xty; 
	double tt = 0; 
	gsl_blas_ddot(xb, yx, &tt);  
	//ytx %*% inv(xtx) xty; 
	double logdetnull = 0; 
	for (int i = 0; i < nc+1; i++)
		logdetnull += log(fabs(gsl_matrix_get(xtx, i, i))); 
	logdetnull *= 2.0; 
	gsl_matrix_free(xtx); 
	gsl_vector_free(yx); 
	gsl_vector_free(xb); 
	//above block is null; 

	double yy = 0; 
	gsl_blas_ddot(gph, gph, &yy); 
	//yty; 
	
	vector<double> vsigma; 
//	vsigma.push_back(0.1); 
	vsigma.push_back(0.2); 
	vsigma.push_back(0.5); 
//	vsigma.push_back(1.0); 

	gsl_vector * bf = gsl_vector_alloc((int)vsigma.size()); 
	for (unsigned p = 0; p < vsigma.size(); p++)
	{
		gsl_matrix_memcpy(ginvOmega, gXtX); 
		double inv_va = 1.0 / vsigma.at(p) / vsigma.at(p); 
		for (int i = 0; i < ns; i++)
		{
			double tmp = gsl_matrix_get(ginvOmega, i+nc+1, i+nc+1) + inv_va; 
			gsl_matrix_set(ginvOmega, i+nc+1, i+nc+1, tmp);
		}   // add prior on diagnal. 
		
		gsl_linalg_cholesky_decomp(ginvOmega); 
		gsl_linalg_cholesky_solve(ginvOmega, gyx, res); 
		//res = inv(xtx+k) %*% yx; 
		double bob = 0;
		gsl_blas_ddot(res, gyx, &bob); 
		//bob = ytx %*% inv(xtx+k) %*% xty; 
		double logdet = 0; 
		for (int i = 0; i < col; i++)
			logdet += log(fabs(gsl_matrix_get(ginvOmega, i, i))); 
		logdet *= 2.0; 
		
//		double tau = sqrt((yy - bob) / ni); 
//		for (int i = 0; i < col; i++)
//			beta[i] += (gsl_vector_get(res, i) / tau); 
        /////////////////////////////////////////////////

		double tlrt = (yy - bob) / (yy - tt);
		double logbf = 0.5*(logdetnull-logdet) - ns*log(vsigma.at(p)) - ni * 0.5 * log(tlrt); 
		gsl_vector_set(bf, p, logbf); 
	}
//	for (int i = 0; i < col; i++)
//		beta[i] /= repeat; 
	double maxbf = gsl_vector_max(bf); 
	gsl_vector_add_constant(bf, -maxbf); 
	double factor = 0; 
	for (size_t i = 0; i < bf->size; i++)
		factor += exp(gsl_vector_get(bf, i)); 
	factor /= (double) bf->size; 
	double logbf = maxbf + log(factor); 
		
	gsl_matrix_free(gX);
	gsl_vector_free(gph);
	gsl_matrix_free(gXtX);
	gsl_matrix_free(ginvOmega);
	gsl_vector_free(gyx);
	gsl_vector_free(res);
	gsl_vector_free(bf); 
	vsigma.clear(); 
	return (logbf);  
}

